==> cl_appbase-0.0.46.tar.gz: lib/src/api/api.dart <==
import 'dart:convert';
import 'dart:typed_data';

import 'package:cl_datahub_common/common.dart';
import 'package:http/http.dart';
import 'package:boost/boost.dart';

import 'api_request_method.dart';
import 'api_response.dart';
import 'utils.dart';

Future<ApiResponse<TResponse>> request<TData, TResponse>(String baseUrl,
    String suffix, Map<String, dynamic>? urlParams, ApiRequestMethod method,
    {Map<String, String>? headers,
    Map<String, String?>? query,
    dynamic body,
    DTOFactory? factory}) async {
  StringBuffer urlBuffer = StringBuffer(baseUrl);
  urlBuffer.write(replacePlaceholders(suffix, urlParams ?? {}));

  if (query?.isNotEmpty ?? false) {
    urlBuffer.write(buildQueryString(query!));
  }

  headers ??= {};
  headers['Content-type'] = 'application/json; charset=UTF-8';

  //TODO better way of object encoding (maybe transfer-encodable interface?)
  var encodedBody;
  if (body is Uint8List) {
    encodedBody = body;
    headers['Content-type'] = 'application/octet-stream;';
  } else if (body is String) {
    encodedBody = body;
  } else if (body is FormData) {
    encodedBody = body.toString();
    headers['Content-type'] = 'application/x-www-form-urlencoded';
  } else {
    encodedBody = JsonEncoder().convert(body);
    headers['Content-type'] = 'application/json; charset=UTF-8';
  }

  final response = await _dispatch(
      method, Uri.parse(urlBuffer.toString()), headers, encodedBody);

  return handleResponse<TData, TResponse>(response, factory);
}

ApiResponse<TResponse> handleResponse<TData, TResponse>(
    Response response, DTOFactory? factory) {
  try {
    if (response.statusCode < 400) {
      final data = _handleData<TData, TResponse>(response.bodyBytes, factory);
      return ApiResponse<TResponse>(response, data);
    } else {
      return ApiResponse(response, null);
    }
  } on Exception catch (e) {
    throw ApiException('Could not process response data.', e);
  }
}

TResponse? _handleData<TData, TResponse>(Uint8List data, DTOFactory? factory) {
  if (TResponse != TData && (!TResponse.isListOfType<TData>())) {
    throw ApiError(
        'Invalid data and response type combination: ${TData.toString()}, ${TResponse.toString()}');
  }

  if (data.isEmpty) {
    return null;
  }

  //TODO encoding from response content-type
  final encoding = utf8;

  if (factory != null) {
    final obj = JsonDecoder().convert(encoding.decode(data));
    if (obj is Map<String, dynamic>) {
      final decodedData = factory(obj) as TData;
      if (TResponse.isList) {
        return [decodedData] as TResponse;
      } else {
        return decodedData as TResponse;
      }
    } else if (obj is List) {
      if (TResponse.isList) {
        return obj.map((e) => factory(e) as TData).toList() as TResponse;
      }
    }

    throw ApiException('Invalid response.');
  } else if (TResponse == String) {
    return encoding.decode(data) as TResponse;
  } else if (TResponse == int) {
    return int.parse(encoding.decode(data)) as TResponse;
  } else if (TResponse == double) {
    return double.parse(encoding.decode(data)) as TResponse;
  } else if (TResponse.isMapOfType<String, dynamic>() ||
      TResponse.isListOfType<dynamic>()) {
    return JsonDecoder().convert(encoding.decode(data)) as TResponse;
  } else if (TResponse == Uint8List) {
    return data as TResponse;
  } else if (TResponse == Null) {
    return null;
  }

  throw ApiError.invalidType(TResponse);
}

Future<Response> _dispatch(ApiRequestMethod type, Uri url,
    Map<String, String>? headers, dynamic body) async {
  switch (type) {
    case ApiRequestMethod.GET:
      return await get(url, headers: headers);
    case ApiRequestMethod.POST:
      return await post(url, headers: headers, body: body);
    case ApiRequestMethod.PUT:
      return await put(url, headers: headers, body: body);
    case ApiRequestMethod.PATCH:
      return await patch(url, headers: headers, body: body);
    case ApiRequestMethod.DELETE:
      return await delete(url, headers: headers);
    default:
      throw ApiError('Invalid request type: $type');
  }
}

==> cl_appbase-0.1.4.tar.gz: lib/src/api/api.dart <==
import 'dart:convert';
import 'dart:typed_data';

import 'package:cl_datahub_common/common.dart';
import 'package:http/http.dart';
import 'package:boost/boost.dart';

import 'api_request_method.dart';
import 'api_response.dart';
import 'utils.dart';

Future<ApiResponse<TResponse>> request<TData, TResponse>(String baseUrl,
    String suffix, Map<String, dynamic>? urlParams, ApiRequestMethod method,
    {Map<String, String>? headers,
    Map<String, String?>? query,
    dynamic body,
    TransferBean<TData>? bean}) async {
  StringBuffer urlBuffer = StringBuffer(baseUrl);
  urlBuffer.write(replacePlaceholders(suffix, urlParams ?? {}));

  if (query?.isNotEmpty ?? false) {
    urlBuffer.write(buildQueryString(query!));
  }

  headers ??= {};
  headers['Content-type'] = 'application/json; charset=UTF-8';

  //TODO better way of object encoding (maybe transfer-encodable interface?)
  var encodedBody;
  if (body is Uint8List) {
    encodedBody = body;
    headers['Content-type'] = 'application/octet-stream;';
  } else if (body is String) {
    encodedBody = body;
  } else if (body is FormData) {
    encodedBody = body.toString();
    headers['Content-type'] = 'application/x-www-form-urlencoded';
  } else if (body != null) {
    encodedBody = JsonEncoder().convert(body);
    headers['Content-type'] = 'application/json; charset=UTF-8';
  }

  final response = await _dispatch(
      method, Uri.parse(urlBuffer.toString()), headers, encodedBody);

  return handleResponse<TData, TResponse>(response, bean);
}

ApiResponse<TResponse> handleResponse<TData, TResponse>(
    Response response, TransferBean<TData>? bean) {
  try {
    if (response.statusCode < 400) {
      final data = _handleData<TData, TResponse>(response.bodyBytes, bean);
      return ApiResponse<TResponse>(response, data);
    } else {
      return ApiResponse(response, null);
    }
  } on Exception catch (e) {
    throw ApiException('Could not process response data.', e);
  }
}

TResponse? _handleData<TData, TResponse>(
    Uint8List data, TransferBean<TData>? bean) {
  if (TResponse != TData && (!TResponse.isListOfType<TData>())) {
    throw ApiError(
        'Invalid data and response type combination: ${TData.toString()}, ${TResponse.toString()}');
  }

  if (data.isEmpty) {
    return null;
  }

  //TODO encoding from response content-type
  final encoding = utf8;

  if (bean != null) {
    final obj = JsonDecoder().convert(encoding.decode(data));
    if (obj is Map<String, dynamic>) {
      final decodedData = bean.toObject(obj);
      if (TResponse.isList) {
        return [decodedData] as TResponse;
      } else {
        return decodedData as TResponse;
      }
    } else if (obj is List) {
      if (TResponse.isList) {
        return obj.map((e) => bean.toObject(e)).toList() as TResponse;
      }
    }

    throw ApiException('Invalid response.');
  } else if (TResponse == String) {
    return encoding.decode(data) as TResponse;
  } else if (TResponse == int) {
    return int.parse(encoding.decode(data)) as TResponse;
  } else if (TResponse == double) {
    return double.parse(encoding.decode(data)) as TResponse;
  } else if (TResponse.isMapOfType<String, dynamic>() ||
      TResponse.isListOfType<dynamic>()) {
    return JsonDecoder().convert(encoding.decode(data)) as TResponse;
  } else if (TResponse == Uint8List) {
    return data as TResponse;
  } else if (TResponse == Null) {
    return null;
  }

  throw ApiError.invalidType(TResponse);
}

Future<Response> _dispatch(ApiRequestMethod type, Uri url,
    Map<String, String>? headers, dynamic body) async {
  switch (type) {
    case ApiRequestMethod.GET:
      return await get(url, headers: headers);
    case ApiRequestMethod.POST:
      return await post(url, headers: headers, body: body);
    case ApiRequestMethod.PUT:
      return await put(url, headers: headers, body: body);
    case ApiRequestMethod.PATCH:
      return await patch(url, headers: headers, body: body);
    case ApiRequestMethod.DELETE:
      return await delete(url, headers: headers);
    default:
      throw ApiError('Invalid request type: $type');
  }
}
